﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Remoting;
using System.Text;
using System.Windows.Forms;
using System.Security.Cryptography;
using System.Drawing;
using System.Configuration;

namespace TapfloDB
{
    public static class Helper
    {
        private static Dictionary<string, View.AbstractView> views = new Dictionary<string, View.AbstractView>(3);
        private static String label = "TapfloDB - "; // String.Format("TapfloDB {0} - ", Server.AppVersion);

        public static View.AbstractView CreateView(string view){
            return CreateView(view, null);
        }
        public static View.AbstractView CreateView(string view, string assembly){
            View.AbstractView v = null;
            if (views.TryGetValue(view, out v)) 
                return v;

            try {
                ObjectHandle oh = Activator.CreateInstance(assembly, "TapfloDB.View." + view + ".Main");
                v = oh.Unwrap() as View.AbstractView;
                v.Anchor = AnchorStyles.Top | AnchorStyles.Left;
                v.Dock = DockStyle.Fill;
                v.Text = label + v.Text; // set view label
                
                views.Add(view, v);
            }
            catch (Exception exp){
                Server.Log("Błąd tworzenia widoku {0} - {1}", v, exp.Message);
                if (exp.InnerException != null) {
                    Server.Log("# {0}\n{1}\n", exp.InnerException.Message, exp.InnerException.StackTrace);
                }
                MessageBox.Show("Wybrana funkcja jest niedostępna!", "Funkcja niedostępna", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }

            return v;
        }

        public static string SHA256(this string password){
            SHA256Managed crypt = new SHA256Managed();
            string hash = String.Empty;
            byte[] crypto = crypt.ComputeHash(Encoding.UTF8.GetBytes(password), 0, Encoding.UTF8.GetByteCount(password));
            foreach (byte bit in crypto){
                hash += bit.ToString("x2");
            }

            return hash;
        }

        public static bool IsSet(this Enum input, Enum matchTo){
            return (Convert.ToUInt32(input) & Convert.ToUInt32(matchTo)) != 0;
        }

        public static void SetupForm(String key, Form form) { 
            Properties.Settings settings = Properties.Settings.Default;
            // Upgrade settings from previous version
            if (settings.Upgrade){
                object prev = settings.GetPreviousVersion("AppVersion");
                Server.Log("Upgrade from {0} to {1}", prev == null ? settings.AppVersion : prev, Server.AppVersion);

                settings.Upgrade();
                settings.Upgrade = false;
                settings.AppVersion = Server.AppVersion.ToString();
                
                // 1.7 RC4 -> 1.7 RC5
                View.TDB.LabelPrinter.LoadLegacySettings();
                
                settings.Save();
            }

            // FormWindowState state = settings.MainWindowState; // cause load...
            FormWindowState state = (FormWindowState)settings[key + "State"];
            Point loc = (Point)settings[key + "Location"];
            Size size = (Size)settings[key + "Size"];
            Rectangle rect = new Rectangle(loc, size);
            int screen = -1;

            // Get screen forced state
            String start = ConfigurationManager.AppSettings[key + "Start"];
            String scr = ConfigurationManager.AppSettings[key + "Screen"];
            if (String.Compare("maximized", start, StringComparison.OrdinalIgnoreCase) == 0){
                state = FormWindowState.Maximized;
                Int32.TryParse(scr, out screen);
            }

            // Set Form Location
            if (state == FormWindowState.Maximized && screen > 0){
                Screen s = Screen.AllScreens[(screen > Screen.AllScreens.Length) ? 0 : screen-1];
                form.StartPosition = FormStartPosition.Manual;
                form.Location = s.WorkingArea.Location;
            }
            else if (size.Width > 300 || size.Height > 300){
                Screen s = Screen.FromRectangle(rect);
                rect = Rectangle.Intersect(s.WorkingArea, rect);
                
                form.StartPosition = FormStartPosition.Manual;
                form.Location = rect.Location;
                form.Size = rect.Size;
            }

            if (state != FormWindowState.Minimized)
                form.WindowState = state;
        }

        public static void SaveForm(String key, Form form){
            Properties.Settings settings = Properties.Settings.Default;

            if (form.WindowState == FormWindowState.Minimized)
                return;
            
            settings[key + "State"] = form.WindowState;

            Rectangle rect = new Rectangle(form.Location, form.Size);
            rect = Rectangle.Intersect(Screen.GetWorkingArea(form), rect);
            if (rect.Width > 300 && rect.Height > 300){
                settings[key + "Location"] = rect.Location;
                if (form.WindowState != FormWindowState.Maximized)
                    settings[key + "Size"] = rect.Size;
            }
            settings.Save();
        }
    }

    [Flags]
    public enum FormMode : int {
        Add = 1,
        Edit = 2,
        Change = 3
    };
}
