﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Configuration;

namespace TapfloDB
{
    public partial class Main : Form
    {
        private ToolStripDropDownItem aitem = null;
        private View.AbstractView view = null;

        #region Initialize

        public Main(){
            InitializeComponent();
        }

        private void Main_Load(object sender, EventArgs e){
#if DEBUG
            Helper.SetupForm("MainWindow", this);
            sprint.DataBindings.Add("Text", new View.TDB.LabelPrinter(), "HeaderMode");
            
            // View.TDB.LabelSettings ls = new View.TDB.LabelSettings();
            // ls.ShowDialog();

            // View.TDB.LabelPrinter lp = new View.TDB.LabelPrinter("TX-01234567890", "1701-0001");
            // View.TDB.LabelPrinter lp = new View.TDB.LabelPrinter("CTI-012345678901234567", "1701-0002");
            // View.TDB.LabelPrinter lp = new View.TDB.LabelPrinter("CTM-01234567890123456789", "1701-0003");
            // View.TDB.LabelPrinter lp = new View.TDB.LabelPrinter("MNK-01234567890", "1701-0002");
            
            // View.TDB.LabelPrinter lp = new View.TDB.LabelPrinter("ALAMARUDEGOKOTA", "1701-0000");
            // lp.Print();
            // Close();
            return;
#endif
            if (!Server.CheckConfiguration()) {
                MessageBox.Show("Nie odnaleziono konfiguracji połaczenia z serverem.\nSkontaktuj się z administratorem w celu skonfigurowania połączenia z serverem.", "Brak konfiguracji połączenia", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
                return;
            }
            if (!Server.CheckConnection()) {
                MessageBox.Show("Nie można nawiązać połączenia z Serwerem\nSprawdz połączenie z siecią lub skontaktuj sie z Administratorem.", "Brak połaczenia z Serverem", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
                return;
            }
            if (Server.AppVersion < Server.DBVersion){
                Server.Log("Wersja aplikacji {0} jest niezgodna z wersją bazy {1}", Server.AppVersion, Server.DBVersion);
            }

#if DEBUG
            Server.Login("admin", "admin1");
#else
            Login l = new Login();
            if (l.ShowDialog(this) != DialogResult.OK){
                Close();
                return;
            }
#endif
            Helper.SetupForm("MainWindow", this);

            Data.User u = Session.Curr.User;

            //Status bar
            suser.Text = u.Name; //Const so no binding needed
            View.TDB.LabelPrinter dummy = new View.TDB.LabelPrinter(); // required for binding
            sprint.DataBindings.Add("Text", dummy, "HeaderMode");
            sstatus.DataBindings.Add("Text", Session.Curr, "LastAction");
            Binding b = new Binding("Text", Session.Curr, "LastUpdate");
            b.Format += new ConvertEventHandler(ssave_Format);
            ssave.DataBindings.Add(b);
            Session.Curr.NotifyAll();
        }

        private void Main_Shown(object sender, EventArgs e){
#if DEBUG
            // to check print button only
            SetView("TDB");
            return;
#endif

            Data.User u = Session.Curr.User;
            
            //Set Active View
            aitem = tTDB;
            
            //Filter Menus
            string defview = u.GetDefaultView();
            if(u.Type != TapfloDB.View.Users.UserType.Admin){
                foreach (ToolStripMenuItem i in GetMenus()){
                    if (!u.Access.Exists(a => (string)a == (string)i.Tag))
                        i.Enabled = i.Visible = false;
                    else if (defview.Equals(i.Tag))
                        aitem = i;
                }

                //Lock Manage if no Admin
                mmanage.Enabled = false;
                mpassword.Visible = false;
            }

            if (defview != null && aitem != null){
                SetView(defview);
                aitem.Image = global::TapfloDB.Properties.Resources.dot;
            }

            ToolStripMenuItem mi = prodDataToolStripMenuItem;
            if (mi.Enabled && String.Compare("true", ConfigurationManager.AppSettings["ProductionDataShowOnStartup"], StringComparison.OrdinalIgnoreCase) == 0){
                menuItem_Click(mi, new EventArgs());
                Focus(); // return focus to main window
            }
        }

        private void ssave_Format(object sender, ConvertEventArgs e){
            DateTime t = (DateTime)e.Value;
            if (t.Year > 1900)
                e.Value = t.ToString();
            else
                e.Value = "";
        }

        #endregion

        #region Closing

        private void mlogout_Click(object sender, EventArgs e){
            Close();
        }

        private void Main_FormClosed(object sender, FormClosedEventArgs e){
            Helper.SaveForm("MainWindow", this);
            Server.Close();
            System.Threading.Thread.Sleep(250);
        }

        #endregion

        #region UI Helpers

        public void SetView(string v){
            SetView(Helper.CreateView(v));
        }
        public void SetView(View.AbstractView c){
            if (c == null) return;

            if (view != null)
                Controls.Remove(view);

            c.Padding = new Padding(5, 36, 5, 36);
            Controls.Add(view = c);
            exportToolStripMenuItem.Enabled = view.SupportExport;
            
            Text = view.Text;
        }

        private ToolStripMenuItem[] GetMenus(){
            var fields = from i in GetType().GetFields(BindingFlags.NonPublic | BindingFlags.Instance)
                         where i.FieldType == typeof(ToolStripMenuItem)
                         select i.GetValue(this) as ToolStripMenuItem;
            return (from i in fields
                    where i != null && (i.OwnerItem == mbazy || i.OwnerItem == mmanage)
                    select i).ToArray();
        }

        #endregion

        #region Menu

        private void exportToolStripMenuItem_Click(object sender, EventArgs e){
            if (view != null)
                view.Export();
            else {
                MessageBox.Show("Funkcja Exportu nie jest dostępna dla tego widoku", "Funkcja niedostępna", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                exportToolStripMenuItem.Enabled = false;
            }
        }

        private void menuItem_Click(object sender, EventArgs e){
            ToolStripDropDownItem mi = sender as ToolStripDropDownItem;
            if (mi == null || mi.Image != null)
                return;

            string tag = mi.Tag as string;
            if (tag == null || tag.Length < 3){
                Server.Log("Nie można załadować widoku: {0}", tag);
                return;
            }

            View.AbstractView c = Helper.CreateView(tag);
            if (c.NewWindow){
                Second s = new Second(c, mi);
                s.Show(this);
            }
            else {
                aitem.Image = null;
                aitem = mi;
                SetView(c);
            }
            mi.Image = global::TapfloDB.Properties.Resources.dot;
        }

        private void zmieńHasłoToolStripMenuItem_Click(object sender, EventArgs e){
            Password p = new Password(FormMode.Edit);
            if (p.ShowDialog() == DialogResult.OK)
                Session.Curr.Action("Pomyślnie ustawiono nowe hasło");
        }

        #endregion

        private void Main_KeyUp(object sender, KeyEventArgs e){
            if (view != null && e.KeyCode == Keys.F5) {
                view.RefreshData();
            }
        }

        private void Main_LogWindowInfo(object sender, EventArgs e){
            Screen s = Screen.FromControl(this);
            Server.Log("Main: {0}x{1}; Size: {2}x{3} ({4}) on {5}", 
                Location.X, Location.Y, Size.Width, Size.Height, WindowState, s.DeviceName);
        }
    }
}
