﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace TapfloDB
{
    public class Session : INotifyPropertyChanged
    {
        #region Singleton

        private static Session _sess = null;

        public static Session Curr {
            get {
                if (!Active)
                    throw new Exception("No active session!");

                return _sess;
            }
        }

        public static bool Active {
            get { return _sess != null; }
        }

        public static bool Start(Data.User u){
            if (Active)
                return false;

            _sess = new Session(u);
            return true;
        }

        #endregion

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName){
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }

        protected bool SetField<T>(ref T field, T value, string propertyName){
            if (EqualityComparer<T>.Default.Equals(field, value)) return false;
            field = value;
            OnPropertyChanged(propertyName);
            return true;
        }

        #endregion

        #region Fields

        private DateTime _ladate;
        private string _laction;
        
        public Data.User User { get; private set; }
        public DateTime Started { get; private set; }
        
        #endregion

        #region Properties

        public DateTime LastUpdate
        {
            get { return _ladate; }
            set { SetField(ref _ladate, value, "LastUpdate"); }
        }
        public string LastAction
        {
            get { return _laction; }
            set { SetField(ref _laction, value, "LastAction"); }
        }

        #endregion

        private Session(Data.User u){
            Started = new DateTime();
            User = u;
        }

        public void Action(String format, params Object[] args){
            string action = String.Format(format, args);
            Server.Log("Status: {0}", action);
            SetField(ref _ladate, DateTime.Now, "LastUpdate");
            SetField(ref _laction, action, "LastAction");
        }

        public void NotifyAll(){
            OnPropertyChanged("LastAction");
            OnPropertyChanged("LastUpdate");
        }

        public void Close(){
            _sess = null;
        }
    }
}
