﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Npgsql;
using System.Data;

namespace TapfloDB.View.Faults
{
    public class Fault
    {
        public int ID { get; private set; }
        public string Name { get; private set; }

        #region Static

        public static Fault Pass { get; private set; }
        public static Fault Fail { get; private set; }
        public static Fault Damage { get; private set; }
        
        public static Fault[] PassFail { get; private set; }
        public static Fault[] FinalResult { get; private set; } 
        public static Fault[] AllFaults { 
            get { return faults.Values.ToArray(); } 
        }

        private static Dictionary<int, Fault> faults = new Dictionary<int, Fault>();

        static Fault(){
            Pass = new Fault(0, "Zaliczony");
            Fail = new Fault(1, "Niezaliczony");
            Damage = new Fault(2, "Uszkodzenie Mechaniczne");
            PassFail = new Fault[] { Pass, Fail };
            FinalResult = new Fault[] { Pass, Fail, Damage };

            Server.Connect();
            NpgsqlDataReader dr = null;
            Fault f;
            try {
                NpgsqlCommand cmd = Server.Command("SELECT id, name FROM tdb.\"TestFailReason\" ORDER BY id;");
                dr = cmd.ExecuteReader();
            
                while (dr.Read()){
                    f = new Fault((int)dr["id"], (string)dr["name"]);
                    faults.Add(f.ID, f);
                }
            }
            catch (Exception exp){
                Server.Log("Błąd inicjalizacji listy Błędów - {0}", exp.Message);
                throw exp;
            }
            finally {
                if(dr != null)
                    dr.Close();
            }
        }

        public static string Find(int id) {
            Fault f = null;
            faults.TryGetValue(id, out f);
            return f != null ? f.Name : null;
        }

        public static Fault Add(int id, string name){
            Fault f = new Fault(id, name);
            if (faults.ContainsKey(id))
                faults.Remove(id); // Remove old instance
            faults.Add(id, f);
            return f;
        }

        #endregion

        public Fault(int id, string name) {
            ID = id;
            Name = name;
        }

        public override string ToString(){
            return Name;
        }
    }
}
