﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OPC.Common;
using OPC.Data.Interface;
using OPC.Data;
using System.Windows.Forms; 

namespace TapfloDB.View.TDB
{
    public class OPC
    {
        #region Fields

        private OpcServer srv;
        private OpcGroup grp;
        private OPCItemDef[] items = new OPCItemDef[2];

        public OPCItemDef[] Items { get; private set; }
        private int[] hItems; //Handles for items on server
        //public OPCItemResult[] ItemResults { get; private set; }
        private OPCItemResult[] itemResults;

        #endregion

        #region Static

        public static string ServerName { get; set; }
        public static string GroupName { get; set; }
        public static string ItemPrefix { get; set; }

        private static OPC _conn = null;
        public static OPC Conn {
            get {
                Connect();
                return _conn;
            }    
        }

        static OPC() {
            ServerName = "S7200.OPCServer";
            GroupName = "Tapflo";
        }

        public static void Connect(){
            if(_conn != null) return;

            _conn = new OPC();
        }
        
        #endregion

        #region Constructor

        private OPC() {
#if DEBUG
            return;
#endif
            srv = new OpcServer();
            srv.Connect(ServerName);

            grp = srv.AddGroup(GroupName, true, 900);
            grp.SetEnable(true);
            // Async read
            //grp.ReadCompleted += grp_ReadComplete;

            Items = ItemsInit();
            grp.AddItems(Items, out itemResults);

            if (itemResults == null)
                throw new Exception("Unamble to add items to group!");

            int faults = 0;
            hItems = new int[Items.Length];
            for (int i = 0; i < itemResults.Length; i++) {
                if (itemResults[i].Error > 0) {
                    hItems[i] = -1;
                    faults++;
                }
                else
                    hItems[i] = itemResults[i].HandleServer;
            }

            if (faults >= Items.Length)
                throw new Exception("All items where added with errors!");
        }

        ~OPC(){
            if (srv != null){
                try{
                    srv.Disconnect();
                }
                catch (Exception ex){
                    // Show friendly error message. 
                    MessageBox.Show(ex.ToString(), "Error");
                }

                srv = null;
            }
        }

        #endregion

        #region Items Definitions

        private static OPCItemDef[] ItemsInit(){
            bool active = true;
            return new OPCItemDef[]{
                new OPCItemDef("1:192.168.5.1:1000:0301,VD0,REAL,R", active, 1, System.Runtime.InteropServices.VarEnum.VT_EMPTY),
                new OPCItemDef("1:192.168.5.1:1000:0301,VD4,REAL,R", active, 2, System.Runtime.InteropServices.VarEnum.VT_EMPTY),
                new OPCItemDef("1:192.168.5.1:1000:0301,V16.4,BOOL,R", active, 3, System.Runtime.InteropServices.VarEnum.VT_EMPTY), // Szczelność mot - Not avalible yet
                new OPCItemDef("1:192.168.5.1:1000:0301,VD8,REAL,R", active, 4, System.Runtime.InteropServices.VarEnum.VT_EMPTY),
                new OPCItemDef("1:192.168.5.1:1000:0301,VD12,REAL,R", active, 5, System.Runtime.InteropServices.VarEnum.VT_EMPTY),
                new OPCItemDef("1:192.168.5.1:1000:0301,V16.0,BOOL,R", active, 6, System.Runtime.InteropServices.VarEnum.VT_EMPTY),
                new OPCItemDef("1:192.168.5.1:1000:0301,V16.1,BOOL,R", active, 7, System.Runtime.InteropServices.VarEnum.VT_EMPTY),
                //new OPCItemDef("1:192.168.5.1:1000:0301,V16.2,BOOL,R", active, 8, System.Runtime.InteropServices.VarEnum.VT_EMPTY),
                new OPCItemDef("1:192.168.5.1:1000:0301,V16.3,BOOL,R", active, 1009, System.Runtime.InteropServices.VarEnum.VT_EMPTY), //Is present flag!!!
                new OPCItemDef("1:192.168.5.1:1000:0301,VW18,INT,R", active, 9, System.Runtime.InteropServices.VarEnum.VT_EMPTY)
            };
        }

        #endregion

        public void AsyncRead(int transaction) { 
            int cancel = -1;
            int[] err = new int[Items.Length];

            grp.AsyncRead(hItems, transaction, out cancel, out err);
        }

        public OPCItemState[] SyncRead(){
            OPCItemState[] states = null;
#if DEBUG
            states = new OPCItemState[]{
                new OPCItemState(){ HandleClient = 1, DataValue = 2.4f, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 2, DataValue = 2.2f, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 3, DataValue = true, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 4, DataValue = 0f, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 5, DataValue = 9.57f, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 6, DataValue = false, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 7, DataValue = false, Error = 0, Quality = 256 },
                //new OPCItemState(){ HandleClient = 8, DataValue = false, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 1009, DataValue = false, Error = 0, Quality = 256 },
                new OPCItemState(){ HandleClient = 9, DataValue = (short)10, Error = 0, Quality = 256 }
            };
#else
            bool res = grp.SyncRead(OPCDATASOURCE.OPC_DS_DEVICE, hItems, out states);
            if (!res || states == null)
                throw new Exception("OPC SyncRead = "+res+", states = " + (states != null ? states.Length.ToString() : "null"));
#endif
            return states;
        }
    }
}
