﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Npgsql;
using System.Text.RegularExpressions;

namespace TapfloDB.View.TestRules
{
    public partial class Main : AbstractView
    {
        private DataTable dt = new DataTable();
        private NpgsqlDataAdapter da;

        public Main(){
            InitializeComponent();
            Text = "Lista Reguł Testów";
        }

        private void Main_Load(object sender, EventArgs e){
            Server.Connect();
            da = Server.DataAdapter("SELECT id, pattern, description FROM tdb.\"TestRules\" WHERE id > 100 ORDER BY id;");
            
            da.Fill(dt);
            dgv.DataSource = dt;
        }

        #region Add/Update

        private void dgv_CellValidating(object sender, DataGridViewCellValidatingEventArgs e){
            if (e.ColumnIndex == 1){
                try {
                    Regex r = new Regex((string)e.FormattedValue);
                }
                catch (ArgumentException ae) {
                    MessageBox.Show("Podana wartość nie jest prawidłowym wyrażeniem regularnym!", "Błędny wzorzec", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    e.Cancel = true;
                }
            }
        }

        private void dgv_CellValueChanged(object sender, DataGridViewCellEventArgs e){
            if (e.RowIndex < 0 || e.ColumnIndex < 1) return;
            NpgsqlCommand cmd;

            // Add
            object id = dgv[0, e.RowIndex].Value;
            if (id == null || id is DBNull){
                object pattern = dgv[1, e.RowIndex].Value;
                if (pattern == null || pattern is DBNull)
                    return;

                try {
                    cmd = Server.Command("INSERT INTO tdb.\"TestRules\" (pattern, description, required) VALUES(:pattern, :description, '{NULL}'); select currval('tdb.\"TestRules_id_seq\"');");
                    cmd.Parameters.AddWithValue("pattern", dgv[1, e.RowIndex].Value);
                    cmd.Parameters.AddWithValue("description", dgv[2, e.RowIndex].Value);
                    id = cmd.ExecuteScalar();
                    if (id != null && (long)id > 0){
                        dgv[0, e.RowIndex].Value = id; // set id to allow deleting
                        Session.Curr.Action("Dodano nową Regułę Testów");
                        TestRule.Add((int)((long)id), (string)dgv[1, e.RowIndex].Value, dgv[2, e.RowIndex].Value as string, new int?[] { null });
                    }
                }
                catch (Exception exp){
                    Server.Log("Błąd dodawania Reguły Testów - {0}", exp.Message);
                    MessageBox.Show("Podczas dodawania nowej Reguły Testów wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany błąd dodawania Reguły Testów", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
            }
            else { // Update
                try {
                    cmd = Server.Command("UPDATE tdb.\"TestRules\" SET pattern = :pattern, description = :description WHERE id = :id;");
                    cmd.Parameters.AddWithValue("id", id);
                    cmd.Parameters.AddWithValue("pattern", dgv[1, e.RowIndex].Value);
                    cmd.Parameters.AddWithValue("description", dgv[2, e.RowIndex].Value);
                    if (cmd.ExecuteNonQuery() > 0)
                        Session.Curr.Action("Zapisano zmiany w Regule Testów {0}", dgv[2, e.RowIndex].Value);
                }
                catch (Exception exp){
                    Server.Log("Błąd modyfikacji Reguły Testów #{0} - {1}", id, exp.Message);
                    MessageBox.Show("Podczas próby modyfikacji Reguły Testów wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany błąd modyfiacji Reguły Testów", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
            }
        }

        #endregion

        #region Remove

        private bool remove(DataGridViewRow row){
            if (row.Cells[0].Value is DBNull) 
                return true; // Not in DB so nothing to do

            NpgsqlCommand cmd = Server.Command("DELETE FROM tdb.\"TestRules\" WHERE id = :id;");
            try {
                int id = (int)row.Cells[0].Value;
                cmd.Parameters.AddWithValue("id", id);
                return cmd.ExecuteNonQuery() > 0;
            }
            catch (Exception exp){
                Server.Log("Błąd usuwania Reguły Testów #{0} - {1}", row.Cells[0].Value, exp.Message);
            }

            return false;
        }
        
        private void dgv_UserDeletingRow(object sender, DataGridViewRowCancelEventArgs e){
            if (dgv.Rows.Count == 0) return;

            if (remove(e.Row))
                Session.Curr.Action("Pomyślnie skasowano Regułe Testów {0}", e.Row.Cells[1].Value);
            else
                MessageBox.Show("Podczas usuwania Reguły Testów wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany błąd usuwania Reguły Testów", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }

        private void bdelete_Click(object sender, EventArgs e){
            int deleted = 0;
            if (dgv.SelectedRows.Count == 1){
                DataGridViewRowCancelEventArgs rce = new DataGridViewRowCancelEventArgs(dgv.SelectedRows[0]);
                dgv_UserDeletingRow(sender, rce);
                if (!rce.Cancel)
                    dgv.Rows.RemoveAt(rce.Row.Index);
            }
            else if (dgv.SelectedRows.Count > 0) {
                foreach (DataGridViewRow r in dgv.SelectedRows){
                    if (remove(r)){
                        dgv.Rows.RemoveAt(r.Index);
                        deleted++;
                    }
                }

                if (deleted == dgv.SelectedRows.Count)
                    Session.Curr.Action("Pomyślnie skasowano {1} Reguł Testów", deleted);
                else
                    Session.Curr.Action("Skasowano {0}/{1} Reguł Testów", deleted, dgv.SelectedRows.Count);
            }
        }

        #endregion

        #region Order Patterns

        /* Not tested!!! 
         * found on:
         * http://www.inforbiro.com/blog-eng/c-sharp-datagridview-drag-and-drop-rows-reorder/
        
        private Rectangle dragBoxFromMouseDown;
        private int rowIndexFromMouseDown;
        private int rowIndexOfItemUnderMouseToDrop;

        private void dataGridView1_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                // If the mouse moves outside the rectangle, start the drag.
                if (dragBoxFromMouseDown != Rectangle.Empty &&
                !dragBoxFromMouseDown.Contains(e.X, e.Y))
                {
                    // Proceed with the drag and drop, passing in the list item.                   
                    DragDropEffects dropEffect = dataGridView1.DoDragDrop(
                          dataGridView1.Rows[rowIndexFromMouseDown],
                          DragDropEffects.Move);
                }
            }
        }

        private void dataGridView1_MouseDown(object sender, MouseEventArgs e)
        {
            // Get the index of the item the mouse is below.
            rowIndexFromMouseDown = dataGridView1.HitTest(e.X, e.Y).RowIndex;

            if (rowIndexFromMouseDown != -1)
            {
                // Remember the point where the mouse down occurred.
                // The DragSize indicates the size that the mouse can move
                // before a drag event should be started.               
                Size dragSize = SystemInformation.DragSize;

                // Create a rectangle using the DragSize, with the mouse position being
                // at the center of the rectangle.
                dragBoxFromMouseDown = new Rectangle(
                          new Point(
                            e.X - (dragSize.Width / 2),
                            e.Y - (dragSize.Height / 2)),
                      dragSize);
            }
            else
                // Reset the rectangle if the mouse is not over an item in the ListBox.
                dragBoxFromMouseDown = Rectangle.Empty;
        }

        private void dataGridView1_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.Move;
        }

        private void dataGridView1_DragDrop(object sender, DragEventArgs e)
        {
            // The mouse locations are relative to the screen, so they must be
            // converted to client coordinates.
            Point clientPoint = dataGridView1.PointToClient(new Point(e.X, e.Y));

            // Get the row index of the item the mouse is below.
            rowIndexOfItemUnderMouseToDrop = dataGridView1.HitTest(clientPoint.X, clientPoint.Y).RowIndex;

            // If the drag operation was a move then remove and insert the row.
            if (e.Effect == DragDropEffects.Move)
            {
                DataGridViewRow rowToMove = e.Data.GetData(typeof(DataGridViewRow)) as DataGridViewRow;
                dataGridView1.Rows.RemoveAt(rowIndexFromMouseDown);
                dataGridView1.Rows.Insert(rowIndexOfItemUnderMouseToDrop, rowToMove);

            }
        }
        */
        #endregion

        private void btest_Click(object sender, EventArgs e){
            TestRule rule = null;
            if (cselected.Checked){
                Regex pattern;
                foreach (DataGridViewRow r in dgv.SelectedRows) { 
                    try {
                        pattern = new Regex((string)r.Cells[1].Value);
                        if (pattern.IsMatch(tproduct.Text)){
                            rule = new TestRule((int)r.Cells[0].Value, (string)r.Cells[1].Value, (string)r.Cells[2].Value, new int?[1]);
                            break;
                        }
                    }
                    catch (ArgumentException ae) {
                    }
                }
            }
            else
                rule = TestRule.Find(tproduct.Text);

            if (rule == null)
                MessageBox.Show(String.Format("Nie znaleziono reguły pasującej do frazy \"{0}\"", tproduct.Text), "Nie znaleziono Reguły Testów", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
                MessageBox.Show(String.Format("Do frazy \"{2}\" pasuje reguła \"{0}\"{1}", rule.Description, rule.ID < 100 ? " (wbudowana)" : "", tproduct.Text), "Znaleziono pasującą Regułę Testów", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}
