﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TapfloDB.View.Users
{
    public partial class AddEdit : Form
    {
        private FormMode mode;
        private bool duplilogin = false;

        private string login = null;

        public bool TypeChange { get; private set; }
        public bool PassChange { get; private set; }
        public Data.User User { get; private set; }

        public AddEdit(FormMode mode, Data.User user){
            this.mode = mode;
            User = user;
            InitializeComponent();
            cviews.Items.AddRange(View.Item.GetViews().ToArray());
            if (mode == FormMode.Edit){
                Text = "Zmień Dane";
                bsave.Text = "Zapisz";
                
                //Set from user
                tlogin.Text = login = user.Login;
                tname.Text = user.Name;
                //tpass.Text = user.EMail;
                
                //Set Type
                if (user.Type == UserType.Admin)
                    radmin.Checked = true;
                else
                    ruser.Checked = true;
                
                //Set Access
                bool all = false;
                View.Item v;
                foreach(String a in user.Access){
                    if (a == "#Full")
                        all = true;

                    for(int i = 0; i < cviews.Items.Count; ++i){
                        v = (View.Item)cviews.Items[i];
                        if(all || v.Tag == a)
                            cviews.SetItemCheckState(i, CheckState.Checked);
                    }

                    if(all) break;
                }
            }
            
            TypeChange = false;
        }

        private void bsave_Click(object sender, EventArgs e){
            if (tlogin.Text.Length < 5){
                MessageBox.Show("Login musi być dłuższy niż 5 znaków!", "Login jest zbyt krótki", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                tlogin.Focus();
                return;
            }

            if (duplilogin){
                MessageBox.Show("Użytkownik o podanym Loginie już istnieje!", "Login jest zajęty", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                tlogin.Focus();
                return;
            }

            if (mode == FormMode.Edit && login != tlogin.Text && MessageBox.Show(String.Format("Czy na pewno chcesz zmienić login użytkownika z {0} na {1}?", login, tlogin.Text), "Zmiana Login'u", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.No){ 
                return;
            }

            if (tname.Text.Length < 5 || tname.Text.IndexOf(' ') == -1){
                MessageBox.Show("Nie podano Imienia i/lub nazwiska", "Podaj Imie i Nazwisko", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                tname.Focus();
                return;
            }

            PassChange = mode == FormMode.Edit && tpass.Text.Length > 0;
            if ((mode != FormMode.Edit || PassChange) && tpass.Text.Length < 6){
                MessageBox.Show("Podane hasło jest zbyt krótkie", "Za krótkie hasło", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                tpass.Focus();
                return;
            }

            if (cviews.CheckedItems.Count == 0) {
                MessageBox.Show("Użytkownik musi mieć do czegoś dostęp!", "Nie ustawiono uprawnień", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            //Set user data
            User.Type = radmin.Checked ? UserType.Admin : UserType.User;
            User.Login = tlogin.Text;
            User.Name = tname.Text;
            User.EMail = tpass.Text;
            User.Access.Clear();
            foreach(View.Item v in cviews.CheckedItems.Cast<View.Item>()){
                if (v.Tag == "#Full")
                    User.Access.Insert(0, v.Tag);
                else
                    User.Access.Add(v.Tag);
            }
            if (User.Access.Count == 1 && User.Access.Contains("#Full")){
                User.Access.Add("TDB");
                User.Access.Add("ProductionData");
            }

            DialogResult = DialogResult.OK;
        }

        private void tlogin_TextChanged(object sender, EventArgs e){
            if (mode == FormMode.Edit && login == tlogin.Text){
                duplilogin = false;
                return;
            }

            Server.Connect();

            string str = (string)Server.Command(String.Format("SELECT login FROM tapflodb.user WHERE lower(login) = '{0}';", tlogin.Text.ToLower())).ExecuteScalar();
            if (duplilogin = str != null)
                tlogin.BorderMode = UI.LoginBoxMode.RedDotted;
            else
                tlogin.BorderMode = UI.LoginBoxMode.Normal;
        }

        private void ruser_CheckedChanged(object sender, EventArgs e){
            if(mode != FormMode.Edit) return;
            
            TypeChange = !TypeChange;
        }
    }
}
