﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Npgsql;
using NpgsqlTypes;

namespace TapfloDB.View.Users
{
    public partial class Main : AbstractView
    {
        private DataTable dt = new DataTable();
        private NpgsqlDataAdapter da;

        public Main(){
            InitializeComponent();
            dgv.AutoGenerateColumns = false;
            Text = "Zarządzaj Użytkownikami";
        }

        private void Main_Load(object sender, EventArgs e){
            Server.Connect();

            Query(true);
            dgv.DataSource = dt;
        }

        #region Filter

        private void tsearch_KeyUp(object sender, KeyEventArgs e){
            if (e.KeyCode == Keys.Return)
                bsearch_Click(sender, e);
        }

        private void tsearch_TextChanged(object sender, EventArgs e){
            if (tsearch.Text.Length == 0)
                Query(false);
        }

        private void bsearch_Click(object sender, EventArgs e){
            int i = Query(true);
            if (i > 0)
                Session.Curr.Action("Znaleziono " + i + " Użytkowników pasujących dla \"" + tsearch.Text + "\"");
            else
                Session.Curr.Action("Nikogo nie znaleziono");
        }

        private int Query(bool filter) {
            string sql = filter && tsearch.Text.Length > 0 ? String.Format(" WHERE name ILIKE '{0}%' OR login ILIKE '{0}%'", tsearch.Text) : null;
            da = Server.DataAdapter(String.Format("SELECT * FROM tapflodb.user{0};", sql));

            try {
                dt.Clear();
                return da.Fill(dt);
            }
            catch (Exception exp){
                Server.Log(filter ? "Błąd filtrowania Użytkowników {0} - {1}" : "Błąd listowania Użytkowników - {1}", sql, exp.Message);
                return -1;
            }
        }

        #endregion

        private void dgv_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e){
            if (e.ColumnIndex == 4){
                string str = dt.Rows[e.RowIndex]["access"] as string;
                if(str.Contains("#Full")){
                    e.Value = "(Pełne)";
                    e.CellStyle.Font = new Font(e.CellStyle.Font, FontStyle.Italic);
                }
                else
                    e.Value = str;
            }
            else if (e.ColumnIndex == 5){
                if(((int)dt.Rows[e.RowIndex]["mode"]) > 0){
                    e.Value = "Aktywny";
                    e.CellStyle.ForeColor = Color.Green;
                }
                else {
                    e.Value = "Zablokowany";
                    e.CellStyle.ForeColor = Color.Red;
                }
            }
        }

        private void badd_Click(object sender, EventArgs e){
            Data.User u = new TapfloDB.Data.User();
            AddEdit a = new AddEdit(FormMode.Add, u);
            if(a.ShowDialog() == DialogResult.OK) {
                NpgsqlConnection conn = Server.Connect();
                NpgsqlTransaction trans = conn.BeginTransaction();
                
                // Acount
                //NpgsqlCommand cmd = Server.Command("INSERT INTO tapflodb.userdata (login, password, name, email, type, access, mode) VALUES (:login, :pass, :name, :email, :type, :access, 3);");
                NpgsqlCommand cmd = Server.Command("INSERT INTO tapflodb.userdata VALUES (DEFAULT, :login, :pass, :name, :email, :type, :access, 1);");
                cmd.Parameters.Add(new NpgsqlParameter("login", NpgsqlDbType.Text));
                cmd.Parameters.Add(new NpgsqlParameter("pass", NpgsqlDbType.Text));
                cmd.Parameters.Add(new NpgsqlParameter("name", NpgsqlDbType.Text));
                cmd.Parameters.Add(new NpgsqlParameter("email", NpgsqlDbType.Text));
                cmd.Parameters.Add(new NpgsqlParameter("type", NpgsqlDbType.Text));
                cmd.Parameters.Add(new NpgsqlParameter("access", NpgsqlDbType.Text));
                
                //Set params
                cmd.Parameters["login"].Value = u.Login;
                cmd.Parameters["pass"].Value = u.EMail.SHA256();
                cmd.Parameters["name"].Value = u.Name;
                cmd.Parameters["email"].Value = u.EMail;
                cmd.Parameters["type"].Value = u.Type;
                cmd.Parameters["access"].Value = String.Join(",", u.Access.ToArray());

                //Role
                NpgsqlCommand role = Server.Command(String.Format("CREATE USER {0} ENCRYPTED PASSWORD '{1}' VALID UNTIL 'infinity' IN ROLE {2};",
                    u.Login, u.EMail, u.Type == UserType.Admin ? "tapflo CREATEROLE" : "tapflouser"));

                try {
                    if (cmd.ExecuteNonQuery() == 1 && role.ExecuteNonQuery() == -1){
                        Session.Curr.Action("Dodano użytkownika " + u.Name);
                        trans.Commit();

                        Query(true);
                    }
                    else {
                        trans.Rollback();
                        Session.Curr.Action("Nie udało się dodać nowego Użytkownika");
                    }
                }
                catch (Exception exp){
                    trans.Rollback();
                    if (exp is NpgsqlException && ((NpgsqlException)exp).Code == "42710"){
                        //BŁĄD: 42710: rola "wojtek1" już istnieje
                        MessageBox.Show("Użytkownik o wybranym loginie już istnieje!\nWybierz inny login i spróbuj ponownie.", "Login już zajęty", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                    else {
                        Server.Log("Błąd dodawania użytkownika - {0}", exp.Message);
                        MessageBox.Show("Podczas dodawania nowego użytkownika wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd dodawania Użytkownika", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                }
            }
        }

        private void bedit_Click(object sender, EventArgs e){
            if(dgv.SelectedRows.Count == 0)
                return;

            DataRowView drv = dgv.SelectedRows[0].DataBoundItem as DataRowView;
            Data.User u = new TapfloDB.Data.User(drv.Row);
            AddEdit a = new AddEdit(FormMode.Edit, u);
            if (a.ShowDialog() == DialogResult.OK){
                NpgsqlConnection conn = Server.Connect();
                NpgsqlTransaction trans = conn.BeginTransaction();
                
                NpgsqlCommand cmd = Server.Command("UPDATE tapflodb.userdata SET " +
                    Server.SetClause("login", "name", "type", "access") +
                    " WHERE id = :id;");
                cmd.Parameters.AddWithValue("id", u.ID);
                cmd.Parameters.AddWithValue("login", u.Login);
                cmd.Parameters.AddWithValue("name", u.Name);
                cmd.Parameters.AddWithValue("type", u.Type);
                cmd.Parameters.AddWithValue("access", String.Join(",", u.Access.ToArray()));

                int r = -1, p = 0;
                try {
                    //Role
                    if (a.TypeChange){
                        NpgsqlCommand role = Server.Command(String.Format(
                            "ALTER USER {0} {1}; REVOKE {2} FROM {0}; GRANT {3} TO {0};",
                            u.Type == UserType.Admin ?
                                new string[]{u.Login, "CREATEROLE", "tapflouser", "tapflo"} : 
                                new string[]{u.Login, "NOCREATEROLE", "tapflo", "tapflouser"}
                        ));
                        r = role.ExecuteNonQuery();
                    }

                    //PassReset
                    if (a.PassChange){
                        NpgsqlCommand pass = Server.Command("SELECT tapflodb.resetpassword(:uid, :pass);");
                        pass.Parameters.AddWithValue("uid", u.ID);
                        pass.Parameters.AddWithValue("pass", u.EMail);
                        p = (int)pass.ExecuteScalar();
                    }

                    if (cmd.ExecuteNonQuery() == 1 && r == -1 && p == 0){
                        trans.Commit();
                        Session.Curr.Action("Pomyślnie zmodyfikowano dane Użytkownika " + u.Name);

                        Query(true);
                    }
                    else {
                        trans.Rollback();
                        Session.Curr.Action("Niemożna zmodyfikowano danych Użytkownika " + u.Name);
                    }
                }
                catch (Exception exp){
                    trans.Rollback();
                    if (exp is NpgsqlException && ((NpgsqlException)exp).Code == "42710"){
                        //BŁĄD: 42710: rola "wojtek1" już istnieje
                        MessageBox.Show("Użytkownik o wybranym loginie już istnieje!\nWybierz inny login i spróbuj ponownie.", "Login już zajęty", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                    else {
                        Server.Log("Błąd edycji użytkownika {0} (#{1}) - {2}", u.Login, u.ID, exp.Message);
                        MessageBox.Show(String.Format("Podczas zmiany danych użytkownika {0} wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", u.Login), "Nieznany Błąd dodawania Użytkownika", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                }
            }
        }

        private void block_Click(object sender, EventArgs e){
            if (dgv.SelectedRows.Count == 0)
                return;

            if (Question("Czy na pewno zmienić stan", "Potwierdz zablokowanie/odblokowanie") == DialogResult.Yes){
                int i = 0;
                foreach (DataGridViewRow r in dgv.SelectedRows){
                    if (Server.Toogle((int)r.Cells["cid"].Value))
                        i++;
                }

                if (i > 0){
                    Session.Curr.Action("Zmieniono stan " + i + " Użytkowników");
                    Query(true);
                }
                else
                    Session.Curr.Action("Nie udało się zmienić stanu wybranych Użytkowników");
            }
        }

        private void bdelete_Click(object sender, EventArgs e){
            if (dgv.SelectedRows.Count == 0)
                return;

            if (Question("Czy na pewno skasować", "Potwierdz usunięcie") == DialogResult.Yes){
                int i = 0;
                foreach (DataGridViewRow r in dgv.SelectedRows){
                    if (Server.Delete((int)r.Cells["cid"].Value, (string)r.Cells["clogin"].Value))
                        i++;
                }

                if (i > 0){
                    Session.Curr.Action("Skasowano " + i + " Użytkowników");
                    Query(true);
                }
                else
                    Session.Curr.Action("Nie można skasować wybranych Użytkowników");
            }
        }

        private DialogResult Question(string text, string caption){
            int i = 0, id = Session.Curr.User.ID;
            StringBuilder sb = new StringBuilder();
            foreach (DataGridViewRow r in dgv.SelectedRows){
                if ((int)r.Cells[0].Value != id){
                    if (i++ > 0) sb.Append(",\n");
                    sb.AppendFormat("{0} ({1})", r.Cells["cname"].Value, r.Cells["clogin"].Value);
                }
            }

            if (i == 0)
                return DialogResult.None;
            if (i == 1)
                text = String.Format("{0} \"{1}\"?", text, sb.ToString());
            else
                text = String.Format("{0} {1} użytkowników?\n{2}", text, i, sb.ToString());

            return MessageBox.Show(text, caption, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
        }
    }
}
